#!/usr/bin/python -u

import multiprocessing as mp
import sys
import os
import re

regex = re.compile(r"\\file +([a-z_]+\.hpp)")


def check(header):
    with open(header) as f:
        lines = f.readlines()
    for l in lines:
        m = regex.search(l)
        if m:
            documented = m.groups()[0]
            basename = os.path.basename(header)
            if documented == basename:
                return 0
            else:
                print(
                    f"Wrong file name ({documented}) in Doxygen \\file directive in {header}",
                    file=sys.stderr,
                )
                return 1

    return 0


if __name__ == "__main__":
    headers = [line.strip() for line in sys.stdin.readlines()]
    errors = sum(check(h) for h in headers)
    sys.exit(errors)



#!/usr/bin/python

import sys

inputs = [
    ("ql.dist.diff", "Some Makefile.am"),
    ("test-suite.dist.diff", "test-suite/Makefile.am"),
    ("ql.cmake.diff", "ql/CMakeLists.txt"),
    ("test-suite.cmake.diff", "test-suite/CMakeLists.txt"),
    ("ql.vcx.diff", "QuantLib.vcxproj"),
    ("ql.vcx.filters.diff", "QuantLib.vcxproj.filters"),
    ("test-suite.vcx.diff", "test-suite/testsuite.vcxproj"),
    ("test-suite.vcx.filters.diff", "test-suite/testsuite.vcxproj.filters"),
]

result = 0


def format(line):
    filename = line[2:].strip()
    if filename.endswith(".hpp"):
        return "header file %s" % filename
    elif filename.endswith(".cpp"):
        return "source file %s" % filename
    else:
        return "file %s" % filename


CYAN = "\033[96m"
RED = "\033[91m"
GREEN = "\033[92m"
BOLD = "\033[1m"
RESET = "\033[0m"

print(BOLD + CYAN + "\n=============================== RESULTS ================================\n" + RESET)

for diffs, target in inputs:
    with open(diffs) as f:
        for line in f:
            if line.startswith("< "):
                print(RED + "%s contains extra %s" % (target, format(line)) + RESET)
                result = 1
            if line.startswith("> "):
                print(RED + "%s doesn't contain %s" % (target, format(line)) + RESET)
                result = 1

if result == 0:
    print(GREEN + "All clear." + RESET)

sys.exit(result)



import re
import sys
import xml.etree.ElementTree as ET
from collections import namedtuple, defaultdict

regex1 = re.compile(".*test_case\(&(.*?)__(.*?)\)")
regex2 = re.compile(".*ext__bind\(&(.*?)__(.*?)__.*\)\)")
regex3 = re.compile(".*{_?([a-zA-Z]*Test)__([a-zA-Z]*?)\(")
regex4 = re.compile("([^.]+)\.(.+)")

def parse_simple(t):
    # test names get mangled - here we extract class and method name
    m = regex1.match(t.attrib["name"])
    if m:
        return m.groups()


def parse_bound(t):
    # same, for cases inside a bind
    m = regex2.match(t.attrib["name"])
    if m:
        return m.groups()


def parse_lambda(t):
    # same, for cases inside a lambda
    m = regex3.match(t.attrib["name"])
    if m:
        return m.groups()

def parse_simple_auto_generated(t):
    # used for classname generated by automated registration using boost test
    m = regex4.match(t.attrib["classname"])
    if m:
        return m.groups()[1], t.attrib["name"]

def extract_test_info(filename):
    root = ET.parse(filename).getroot()
    tests = root.findall("testcase")
    for t in tests:
        if t.findall("skipped"):
            continue
        cls, method = parse_simple(t) or parse_bound(t) or parse_lambda(t) or parse_simple_auto_generated(t)
        time = float(t.attrib["time"])
        yield (cls, method, time)


Case = namedtuple("Case", ["files", "times"])


def collect():
    data = defaultdict(lambda: Case([], []))
    for filename in ["faster.xml", "fast.xml", "all.xml"]:
        for cls, method, time in extract_test_info(filename):
            c = data[(cls, method)]
            c.files.append(filename)
            c.times.append(time)
    return data


def check(data, condition, action):
    errors = False
    for cls, method in data:
        c = data[(cls, method)]
        if condition(c):
            print(
                f"{cls}.{method} ({min(c.times):.2f} s / {max(c.times):.2f} s) {action}"
            )
            errors = True
    return errors


data = collect()

errors = check(
    data,
    lambda c: "faster.xml" in c.files and min(c.times) > 3.0,
    "should be moved out of the 'Faster' section",
)
errors |= check(
    data,
    lambda c: "fast.xml" in c.files and min(c.times) > 8.0,
    "should be moved out of the 'Fast' section",
)
errors |= check(
    data, lambda c: max(c.times) > 30.0, "should be kept reliably under 30 seconds"
)

errors |= check(
    data,
    lambda c: "faster.xml" not in c.files and max(c.times) < 1.0,
    "could be moved into the 'Faster' section",
)
errors |= check(
    data,
    lambda c: "fast.xml" not in c.files and max(c.times) < 3.0,
    "could be moved into the 'Fast' section",
)


if errors:
    sys.exit(1)



#!/usr/bin/python -u

import multiprocessing as mp
import os
import sys
import shutil


def check(header):
    cxx = os.environ.get("CXX", "g++")
    source_file = header + ".cpp"
    shutil.copy(header, source_file)
    object_file = header + ".o"
    command = f"{cxx} -std=c++14 -c -Wno-unknown-pragmas -Wall -Werror -I. {source_file} -o {object_file}"
    print(command, file=sys.stderr)
    code = os.system(command)
    try:
        os.remove(source_file)
        os.remove(object_file)
    except:
        pass
    if code != 0:
        print(f"Errors while checking {header}", file=sys.stderr)
        return 1
    return 0


if __name__ == "__main__":
    headers = [line.strip() for line in sys.stdin.readlines()]
    with mp.Pool(processes=mp.cpu_count()) as pool:
        errors = sum(pool.map(check, headers))
    sys.exit(errors)



#!/usr/bin/python3

import sys, re

if len(sys.argv) != 2:
    print(f"Usage: {sys.argv[0]} <file>")
    sys.exit()

filename = sys.argv[1]
print(f"Checking {filename}")

regex = re.compile("^#include.*<(.*)>")
ql_headers = []
boost_headers = []
std_headers = []
experimental_headers = []
source = open(filename)
for n, line in enumerate(source):
    match = regex.search(line)
    if match:
        (header,) = match.groups()
        line_num = n + 1
        if header.startswith("ql/"):
            ql_headers.append((header, line_num))
            if header.startswith("ql/experimental") and "internal" not in line:
                experimental_headers.append((header, line_num))
        elif header.startswith("boost/"):
            boost_headers.append((header, line_num))
        else:
            std_headers.append((header, line_num))
source.close()

# At least one QuantLib header muct be included (which, hopefully,
# ultimately leads to including ql/qldefines.hpp)
if not ql_headers:
    print(f"./{filename}:1: error: no QuantLib header included")
    sys.exit(1)

if "ql/experimental" not in filename:
    # files in core library can't include stuff in experimental
    if experimental_headers:
        for f, n in experimental_headers:
            print(f"./{filename}:{n}: error: experimental header '{f}' included")
        sys.exit(1)

for f, n in ql_headers:
    if f.endswith("/all.hpp"):
        print(f"./{filename}:{n}: error: generated header '{f}' included")
        sys.exit(1)

# All Boost headers must be included after QuantLib ones
last_ql_header = max([n for _, n in ql_headers])

for _, n in boost_headers:
    if n < last_ql_header:
        print(
            f"./{filename}:{n}: error: Boost header included before last QuantLib header"
        )
        sys.exit(1)


# All standard headers must be included after QuantLib and Boost ones
if boost_headers:
    last_boost_header = max([n for _, n in boost_headers])

for _, n in std_headers:
    if n < last_ql_header:
        print(
            f"./{filename}:{n}: error: standard header included before last QuantLib header"
        )
        sys.exit(1)
    if boost_headers and n < last_boost_header:
        print(
            f"./{filename}:{n}: error: standard header included before last Boost header"
        )
        sys.exit(1)
